@@ -6,6 +6,9 @@ var temp_chart_title;
 var currentChart = null;
 var currentSettings = null;
 
+var dateTimeCols = [];
+var numericCols = [];
+
 function extractDate(dateString) {
     var matches, match;
     var dateTimeRegExp = /[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}/;
@@ -53,12 +56,7 @@ function PMA_queryChart(data, columnNames, settings) {
                 label : settings.yaxisLabel
             }
         },
-        stackSeries : settings.stackSeries,
-        highlighter: {
-            show: true,
-            showTooltip: true,
-            tooltipAxes: 'xy'
-        }
+        stackSeries : settings.stackSeries
     };
 
     // create the chart
@@ -69,6 +67,8 @@ function PMA_queryChart(data, columnNames, settings) {
     var dataTable = new DataTable();
     if (settings.type == 'timeline') {
         dataTable.addColumn(ColumnType.DATE, columnNames[settings.mainAxis]);
+    } else if (settings.type == 'scatter') {
+        dataTable.addColumn(ColumnType.NUMBER, columnNames[settings.mainAxis]);
     } else {
         dataTable.addColumn(ColumnType.STRING, columnNames[settings.mainAxis]);
     }
@@ -90,6 +90,8 @@ function PMA_queryChart(data, columnNames, settings) {
             if (j === 0) {
                 if (settings.type == 'timeline') { // first column is date type
                     newRow.push(extractDate(row[col]));
+                } else if (settings.type == 'scatter') {
+                    newRow.push(parseFloat(row[col]));
                 } else { // first column is string type
                     newRow.push(row[col]);
                 }
@@ -110,7 +112,7 @@ function drawChart() {
     currentSettings.width = $('#resizer').width() - 20;
     currentSettings.height = $('#resizer').height() - 20;
 
-    // todo: a better way using .redraw() ?
+    // TODO: a better way using .redraw() ?
     if (currentChart !== null) {
         currentChart.destroy();
     }
@@ -135,6 +137,51 @@ function getSelectedSeries() {
     return ret;
 }
 
+function onXAxisChange() {
+    var $xAxisSelect = $('select[name="chartXAxis"]');
+    currentSettings.mainAxis = parseInt($xAxisSelect.val(), 10);
+    if (dateTimeCols.indexOf(currentSettings.mainAxis) != -1) {
+        $('span.span_timeline').show();
+    } else {
+        $('span.span_timeline').hide();
+        if (currentSettings.type == 'timeline') {
+            $('input#radio_line').prop('checked', true);
+            currentSettings.type = 'line';
+        }
+    }
+    if (numericCols.indexOf(currentSettings.mainAxis) != -1) {
+        $('span.span_scatter').show();
+    } else {
+        $('span.span_scatter').hide();
+        if (currentSettings.type == 'scatter') {
+            $('input#radio_line').prop('checked', true);
+            currentSettings.type = 'line';
+        }
+    }
+    var xaxis_title = $xAxisSelect.children('option:selected').text();
+    $('input[name="xaxis_label"]').val(xaxis_title);
+    currentSettings.xaxisLabel = xaxis_title;
+}
+
+function onDataSeriesChange() {
+    var $seriesSelect = $('select[name="chartSeries"]');
+    currentSettings.selectedSeries = getSelectedSeries();
+    var yaxis_title;
+    if (currentSettings.selectedSeries.length == 1) {
+        $('span.span_pie').show();
+        yaxis_title = $seriesSelect.children('option:selected').text();
+    } else {
+        $('span.span_pie').hide();
+        if (currentSettings.type == 'pie') {
+            $('input#radio_line').prop('checked', true);
+            currentSettings.type = 'line';
+        }
+        yaxis_title = PMA_messages.strYValues;
+    }
+    $('input[name="yaxis_label"]').val(yaxis_title);
+    currentSettings.yaxisLabel = yaxis_title;
+}
+
 /**
  * Unbind all event handlers before tearing down a page
  */
@@ -181,21 +228,20 @@ AJAX.registerOnload('tbl_chart.js', function () {
 
     // handle chart type changes
     $('input[name="chartType"]').click(function () {
-        currentSettings.type = $(this).val();
-        drawChart();
-        if ($(this).val() == 'bar' || $(this).val() == 'column' ||
-            $(this).val() == 'line' || $(this).val() == 'area' ||
-            $(this).val() == 'timeline' || $(this).val() == 'spline'
-        ) {
+        var type = currentSettings.type = $(this).val();
+        if (type == 'bar' || type == 'column' || type == 'area') {
             $('span.barStacked').show();
         } else {
+            $('input[name="barStacked"]').attr('checked', false);
+            $.extend(true, currentSettings, {stackSeries : false});
             $('span.barStacked').hide();
         }
+        drawChart();
     });
 
     // handle stacking for bar, column and area charts
     $('input[name="barStacked"]').click(function () {
-        if (this.checked) {
+        if ($(this).is(':checked')) {
             $.extend(true, currentSettings, {stackSeries : true});
         } else {
             $.extend(true, currentSettings, {stackSeries : false});
@@ -204,62 +250,43 @@ AJAX.registerOnload('tbl_chart.js', function () {
     });
 
     // handle changes in chart title
-    $('input[name="chartTitle"]').focus(function () {
+    $('input[name="chartTitle"]')
+    .focus(function () {
         temp_chart_title = $(this).val();
-    }).keyup(function () {
+    })
+    .keyup(function () {
         var title = $(this).val();
         if (title.length === 0) {
             title = ' ';
         }
         currentSettings.title = $('input[name="chartTitle"]').val();
         drawChart();
-    }).blur(function () {
+    })
+    .blur(function () {
         if ($(this).val() != temp_chart_title) {
             drawChart();
         }
     });
 
-    var dateTimeCols = [];
     var vals = $('input[name="dateTimeCols"]').val().split(' ');
     $.each(vals, function (i, v) {
         dateTimeCols.push(parseInt(v, 10));
     });
 
+    var vals = $('input[name="numericCols"]').val().split(' ');
+    $.each(vals, function (i, v) {
+        numericCols.push(parseInt(v, 10));
+    });
+
     // handle changing the x-axis
     $('select[name="chartXAxis"]').change(function () {
-        currentSettings.mainAxis = parseInt($(this).val(), 10);
-        if (dateTimeCols.indexOf(currentSettings.mainAxis) != -1) {
-            $('span.span_timeline').show();
-        } else {
-            $('span.span_timeline').hide();
-            if (currentSettings.type == 'timeline') {
-                $('input#radio_line').prop('checked', true);
-                currentSettings.type = 'line';
-            }
-        }
-        var xaxis_title = $(this).children('option:selected').text();
-        $('input[name="xaxis_label"]').val(xaxis_title);
-        currentSettings.xaxisLabel = xaxis_title;
+        onXAxisChange();
         drawChart();
     });
 
     // handle changing the selected data series
     $('select[name="chartSeries"]').change(function () {
-        currentSettings.selectedSeries = getSelectedSeries();
-        var yaxis_title;
-        if (currentSettings.selectedSeries.length == 1) {
-            $('span.span_pie').show();
-            yaxis_title = $(this).children('option:selected').text();
-        } else {
-            $('span.span_pie').hide();
-            if (currentSettings.type == 'pie') {
-                $('input#radio_line').prop('checked', true);
-                currentSettings.type = 'line';
-            }
-            yaxis_title = PMA_messages.strYValues;
-        }
-        $('input[name="yaxis_label"]').val(yaxis_title);
-        currentSettings.yaxisLabel = yaxis_title;
+        onDataSeriesChange();
         drawChart();
     });
 
@@ -273,6 +300,9 @@ AJAX.registerOnload('tbl_chart.js', function () {
         drawChart();
     });
 
+    onXAxisChange();
+    onDataSeriesChange();
+
     $("#tblchartform").submit();
 });
 
@@ -315,8 +345,6 @@ $("#tblchartform").live('submit', function (event) {
         } else {
             PMA_ajaxRemoveMessage($msgbox);
             PMA_ajaxShowMessage(data.error, false);
-            chart_data = null;
-            drawChart();
         }
         PMA_ajaxRemoveMessage($msgbox);
     }, "json"); // end $.post()
