@@ -10,7 +10,7 @@ if (! defined('PHPMYADMIN')) {
 }
 
 require_once 'libraries/Scripts.class.php';
-require_once 'libraries/RecentTable.class.php';
+require_once 'libraries/RecentFavoriteTable.class.php';
 require_once 'libraries/Menu.class.php';
 require_once 'libraries/navigation/Navigation.class.php';
 require_once 'libraries/url_generating.lib.php';
@@ -27,14 +27,14 @@ class PMA_Header
      * PMA_Scripts instance
      *
      * @access private
-     * @var object
+     * @var PMA_Scripts
      */
     private $_scripts;
     /**
      * PMA_Menu instance
      *
      * @access private
-     * @var object
+     * @var PMA_Menu
      */
     private $_menu;
     /**
@@ -479,6 +479,7 @@ class PMA_Header
                 . $GLOBALS['cfg']['CSPAllow'] . ';'
                 . "script-src 'self' 'unsafe-inline' 'unsafe-eval' "
                 . ($use_captcha ? 'https://www.google.com ' : ' ')
+                . $GLOBALS['cfg']['CSPAllow'] . ';'
                 . ";"
                 . "style-src 'self' 'unsafe-inline' "
                 . ($use_captcha ? 'https://www.google.com ' : ' ')
@@ -577,7 +578,7 @@ class PMA_Header
         $retval .= '<meta name="robots" content="noindex,nofollow" />';
         $retval .= '<meta http-equiv="X-UA-Compatible" content="IE=Edge">';
         if (! $GLOBALS['cfg']['AllowThirdPartyFraming']) {
-            $retval .= '<style>html{display: none;}</style>';
+            $retval .= '<style id="cfs-style">html{display: none;}</style>';
         }
         return $retval;
     }
@@ -683,7 +684,7 @@ class PMA_Header
         if ($this->_warningsEnabled) {
             $retval .= "<noscript>";
             $retval .= PMA_message::error(
-                __("Javascript must be enabled past this point")
+                __("Javascript must be enabled past this point!")
             )->getDisplay();
             $retval .= "</noscript>";
         }
@@ -705,7 +706,7 @@ class PMA_Header
             && strlen($table)
             && $GLOBALS['cfg']['NumRecentTables'] > 0
         ) {
-            $tmp_result = PMA_RecentTable::getInstance()->add($db, $table);
+            $tmp_result = PMA_RecentFavoriteTable::getInstance('recent')->add($db, $table);
             if ($tmp_result === true) {
                 $params  = array('ajax_request' => true, 'recent_table' => true);
                 $url     = 'index.php' . PMA_URL_getCommon($params);
